from .fulfillment_automation import FulfillmentAutomation
from .fulfillment import FulfillmentResource
from .template import TemplateResource


__all__ = [
    'FulfillmentAutomation',
    'FulfillmentResource',
    'TemplateResource'
]
